<?php

/**
 * Mega Forms Form Settings
 *
 * @link       https://wpali.com
 * @since      1.0.3
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/admin/partials/views/form-views
 */
if (!defined('ABSPATH')) {
  exit; // Exit if accessed directly
}

class MegaForms_Form_Settings
{

  /**
   * Holds the form object ( contains form id and settings as well )
   *
   * @var object
   */
  private $form;

  /**
   * Hold the form ID
   *
   * @var int
   */
  private $form_id;
  /**
   * Holds the settings values
   *
   * @var array
   */
  private $settings;

  public function __construct($form = '')
  {

    if (!empty($form) && is_object($form)) {
      // Set form object
      $this->form = $form;
      // Set form id
      $this->form_id = $form->ID;
      // Set form settings
      $this->settings = $form->settings;
    }
  }
  /**
   * Returns the default settings tabs
   *
   */
  public function get_option_tabs()
  {

    return apply_filters('mf_form_settings_tabs', array(
      'general' => __('General', 'megaforms'),
      'restrictions' => __('Restrictions', 'megaforms'),
      'confirmation' => __('Confirmation', 'megaforms'),
    ));
  }
  /**
   * Returns list of options for each tab with their related arguements
   *
   * @return array
   */
  public function get_options()
  {

    $pages = get_pages(array('post_type' => 'page'));
    $pages_array = array();
    foreach ($pages as $page) {
      $pages_array[$page->ID] = $page->post_title;
    }
    # Assign callback functions to each tab ( a callback for every option )
    $options = array(
      'general' => array(
        'form_description' => array(
          'type' => 'textarea',
          'label' => __('Form Description', 'megaforms'),
          'value' => mfget('form_description', $this->settings),
          'sanitization' => 'string',
        ),
        'submit_button' => array(
          'type' => 'text',
          'label' => __('Submit Text', 'megaforms'),
          'value' => mfget('submit_button', $this->settings, __('Submit', 'megaforms')),
          'sanitization' => 'string',
        ),
        'submit_button_processing' => array(
          'type' => 'text',
          'label' => __('Submit processing Text', 'megaforms'),
          'value' => mfget('submit_button_processing', $this->settings, __('Processing…', 'megaforms')),
          'sanitization' => 'string',
        ),
        'form_css_class' => array(
          'type' => 'text',
          'label' => __('CSS Class', 'megaforms'),
          'desc' => __('Add custom CSS class names here to use on your form wrappers. Multiple classes should be separated by space.', 'megaforms'),
          'value' => mfget('form_css_class', $this->settings),
          'sanitization' => 'string',
        ),
        'disable_storing_entries' => array(
          'type' => 'switch',
          'label' => __('Stop Entries', 'megaforms'),
          'value' => mfget('disable_storing_entries', $this->settings),
          'desc' => __('Enable this option to prevent entries from being stored in the database.', 'megaforms'),
          'sanitization' => 'boolean',
        ),
      ),
      'restrictions' => array(
        'time_trap' => array(
          'type' => 'switch',
          'label' => __('Timetrap ( Anti-Spam )', 'megaforms'),
          'value' => mfget('time_trap', $this->settings, true),
          'desc' => __('Enable time trap technique to help with spam prevention. Spam bots usually fills out forms very fast. The timetrap prevents form submission until a number of seconds has passed.', 'megaforms'),
          'sanitization' => 'boolean',
        ),
        'time_trap_duration' => array(
          'type' => 'number',
          'label' => __('Timetrap Seconds', 'megaforms'),
          'desc' => __('The minimum time after which a normal user can submit the form (in seconds).', 'megaforms'),
          'value' => mfget('time_trap_duration', $this->settings, 5),
          'placeholder' => 5,
          'parent' => 'time_trap',
          'parent_value' => '1',
          'sanitization' => 'integer',
        ),
        'limited_entries' => array(
          'type' => 'switch',
          'label' => __('Limited Entries', 'megaforms'),
          'value' => mfget('limited_entries', $this->settings),
          'desc' => __('Enable this option to allow a specific number of entries for this form that can not be exceeded.', 'megaforms'),
          'sanitization' => 'boolean',
        ),
        'form_submission_limit' => array(
          'type' => 'number',
          'label' => __('Submissions Limit', 'megaforms'),
          'value' => mfget('form_submission_limit', $this->settings),
          'parent' => 'limited_entries',
          'parent_value' => '1',
          'sanitization' => 'integer',
        ),
        'limit_reached_msg' => array(
          'type' => 'textarea',
          'label' => __('Submissions Limit', 'megaforms'),
          'value' => mfget('limit_reached_msg', $this->settings),
          'parent' => 'limited_entries',
          'parent_value' => '1',
          'sanitization' => 'string',
        ),
        'login_restricted' => array(
          'type' => 'switch',
          'label' => __('Login Restricted', 'megaforms'),
          'value' => mfget('login_restricted', $this->settings),
          'desc' => __('Enable this option to require login to view the form.', 'megaforms'),
          'sanitization' => 'boolean',
        ),
        'login_restricted_msg' => array(
          'type' => 'textarea',
          'label' => __('Require Login Message', 'megaforms'),
          'value' => mfget('login_restricted_msg', $this->settings),
          'parent' => 'login_restricted',
          'parent_value' => '1',
          'sanitization' => 'string',
        ),
      ),
      'confirmation' => array(
        'confirmation_type' => array(
          'type' => 'radio',
          'label' => __('Confirmation Type', 'megaforms'),
          'value' => mfget('confirmation_type', $this->settings, 'message'),
          'desc' => sprintf(
            '%s<br></br><b>%s</b>%s<br></br><b>%s</b>%s<br></br><b>%s</b>%s',
            __('Select confirmation type for this form. This will be the result after the user submits the form.', 'megaforms'),
            __('Message: ', 'megaforms'),
            __('The text users will see after submitting the form.', 'megaforms'),
            __('Page: ', 'megaforms'),
            __('The page users will be redirected to after submitting the form.', 'megaforms'),
            __('Redirection: ', 'megaforms'),
            __('The URL of webpage users will be redirected to after submitting the form.', 'megaforms')
          ),
          'options' => array(
            'message' => __('Message', 'megaforms'),
            'page' => __('Page', 'megaforms'),
            'redirect' => __('Redirect', 'megaforms'),
          ),
          'sanitization' => 'string',
        ),
        'confirmation_message' => array(
          'type' => 'textarea',
          'label' => __('Confirmation Message', 'megaforms'),
          'value' => mfget('confirmation_message', $this->settings, __('Form has been successfully submitted. Thank you.', 'megaforms')),
          'parent' => 'confirmation_type',
          'parent_value' => 'message',
          'sanitization' => 'string',
        ),
        'keep_form' => array(
          'type' => 'switch',
          'label' => __('Keep Form', 'megaforms'),
          'value' => mfget('keep_form', $this->settings, false),
          'desc' => __('This option allows you to keep or hide the form fields after successful submission.', 'megaforms'),
          'parent' => 'confirmation_type',
          'parent_value' => 'message',
          'sanitization' => 'boolean',
        ),
        'confirmation_page' => array(
          'type' => 'select',
          'label' => __('Confirmation Page', 'megaforms'),
          'value' => mfget('confirmation_page', $this->settings, 0),
          'options' => $pages_array,
          'parent' => 'confirmation_type',
          'parent_value' => 'page',
          'sanitization' => 'string',
        ),
        'confirmation_redirect' => array(
          'type' => 'url',
          'label' => __('Confirmation Redirect URL', 'megaforms'),
          'value' => mfget('confirmation_redirect', $this->settings),
          'parent' => 'confirmation_type',
          'parent_value' => 'redirect',
          'sanitization' => 'url',
        ),
      ),
    );

    return apply_filters('mf_form_settings_options', $options, $this->settings);
  }

  /**
   * Render form settings on the backend.
   *
   * @since    1.0.0
   */
  public function render()
  {
?>
    <div id="mega-body" class="metabox-holder columns-2">

      <div id="mega-settings-container" class="mega-container">
        <div id="megaforms_settings">
          <div class="clearfix"></div>
          <?php
          $tabs = $this->get_option_tabs();
          $i = 0;
          foreach ($tabs as $tabKey => $tabName) {
            $active = $i === 0 ? ' active' : '';
          ?>
            <a href="#" title="<?php echo $tabName; ?>" data-panel="<?php echo $tabKey; ?>_settings" data-disable="mgform_settings" class="mf-panel-toggler<?php echo $active ?>" target="_self"><span class="megaforms-menu-text"><?php echo $tabName; ?></span></a>
          <?php
            $i++;
          }
          ?>
          <div class="clearfix"></div>
        </div>
      </div>

      <div id="mega-body-settings" class="mega-body-content">
        <form data-id="<?php echo $this->form_id; ?>" method="post" id="mform_settings" class="megaforms-settings">
          <ul data-id="<?php echo $this->form_id; ?>" id="mgsettingssholder">
            <?php
            $tabs = $this->get_options();
            $i = 0;
            foreach ($tabs as $tab => $options) {
              $active = $i === 0 ? ' active' : '';
            ?>
              <li id="<?php echo $tab; ?>_settings" class="mgform_settings<?php echo $active; ?>">
                <?php
                foreach ($options as $name => $args) {

                  $attrs = array();
                  $attrs['name'] = $name;
                  $attrs['id'] = $name;
                  if (isset($args['class'])) $attrs['class'] = $args['class'];
                  if (isset($args['placeholder'])) $attrs['placeholder'] = $args['placeholder'];
                  if (isset($args['value']) && ($args['type'] !== 'radio' && $args['type'] !== 'checkbox')) $attrs['value'] = esc_attr($args['value']);

                  if (isset($args['parent'])) {

                    $attrs['data-conditional-rules'] = json_encode(array(
                      'container' => '.sub-setting',
                      'rules' => array(
                        'name' => $args['parent'],
                        'operator' => 'is',
                        'value' => $args['parent_value'],
                      ),
                    ));
                ?>
                    <table class="mgform_<?php echo $tab; ?>_settings sub-setting" cellspacing="0" cellpadding="0">
                      <tbody>
                      <?php
                    } else {
                      ?>
                        <table class="mgform_<?php echo $tab; ?>_settings" cellspacing="0" cellpadding="0">
                          <tbody>
                          <?php
                        }
                          ?>
                          <tr>
                            <th><label for='<?php echo $name ?>'></label><?php echo $args['label'] ?>
                              <?php
                              if (isset($args['desc'])) {
                                echo $this->get_description_tip_markup($args['label'], $args['desc']);
                              }
                              ?>
                            </th>
                            <td class='mf-inner-field'>
                              <?php
                              if (isset($args['before_field'])) {
                                echo $args['before_field'];
                              }
                              ?>
                              <?php switch ($args['type']) {
                                case 'switch':
                                  if (mfget_bool_value($args['value'])) $attrs['checked'] = 'checked';
                                  echo get_mf_switch($attrs);
                                  break;
                                case 'textarea':
                                  echo get_mf_textarea($attrs);
                                  break;
                                case 'radio':
                                case 'checkbox':
                                  if (isset($args['options'])) {
                                    foreach ($args['options'] as $key => $title) {
                                      $attrs['id'] = $attrs['id'] . '_' . $key;
                                      $attrs['value'] = $key;
                                      if ($args['value'] == $key) $attrs['checked'] = 'checked';
                                      echo get_mf_input($args['type'], $attrs);
                                      if (isset($attrs['checked']) == $key) unset($attrs['checked']);
                              ?>
                                      <label for="<?php echo $attrs['id']; ?>"><?php echo $title; ?></label>
                              <?php
                                    }
                                  }
                                  unset($attrs['value']);
                                  break;
                                case 'select':
                                  if (isset($args['options'])) {
                                    echo get_mf_select($attrs, $args['options']);
                                  }
                                  break;
                                case 'custom':
                                  if (isset($args['content'])) {
                                    echo $args['content'];
                                  }
                                  break;
                                default:
                                  echo get_mf_input($args['type'], $attrs);
                                  break;
                              } ?>
                              <?php
                              if (isset($args['after_field'])) {
                                echo $args['after_field'];
                              }
                              ?>
                            </td>
                          </tr>
                          </tbody>
                        </table>
                      <?php
                    }
                      ?>
              </li>
            <?php
              $i++;
            }
            ?>
          </ul>
        </form>
      </div>
    </div>
<?php
  }

  /**********************************************************************
   ***************************** Helpers ********************************
   **********************************************************************/
  /**
   *  Return description tooltip markup.
   *
   */
  private function get_description_tip_markup($label, $content)
  {
    $html = "";
    $html .= "<span class='mf-tooltip settings-tooltip mega-icons-question-circle'>";
    $html .= "<span style='display:none;' id='tooltip-content'><h4>" . $label . "</h4>" . $content . "</span>";
    $html .= "</span>";

    return $html;
  }
}
