<?php

/**
 * @link       https://wpali.com
 * @since      1.0.3
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/common/partials/form/actions
 */

/**
 * Load actions and handle registration
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/common/partials/form/actions
 * @author     ALI KHALLAD <ali@wpali.com>
 */

if (!defined('ABSPATH')) {
  exit; // Exit if accessed directly
}

class MF_Containers
{

  public  $editable         = false;
  public  $type             = '';

  public function __construct($type, $is_editor = false)
  {

    # Define properties
    if ($is_editor === true) {
      $this->editable = true;
    }

    $available_types = $this->get_allowed_container_types();
    if (isset($available_types[$type])) {
      $this->type = $type;
    } else {
      return false;
    }
  }

  /**
   *  Get the container allowed types.
   *
   */
  public function get_allowed_container_types()
  {
    return apply_filters('mf_fields_container_types', array(
      'row' => array(
        'name' => esc_attr__('Row', 'megaforms'),
        'icon' => 'dashicons-grid-view',
      ),
      'repeatable' => array(
        'name' => esc_attr__('Repeatable', 'megaforms'),
        'icon' => 'dashicons-feedback',
        'options' => array('title', 'key'),
      ),
      'step' => array(
        'name' => esc_attr__('Steps', 'megaforms'),
        'icon' => 'dashicons-excerpt-view',
        'options' => array('title'),
      ),
    ));
  }

  /**
   *  Get the container title.
   *
   */
  public function get_container_title()
  {

    $allowed_types = $this->get_allowed_container_types();
    return $allowed_types[$this->type]['name'];
  }

  /**
   *  Get the container icon.
   *
   */
  public function get_container_icon()
  {

    $allowed_types = $this->get_allowed_container_types();
    return $allowed_types[$this->type]['icon'];
  }

  /**
   *  Get the container for editor view.
   *
   * @since    1.0.0
   * @param string $additional_class
   * @return string
   */
  public function get_single_wrapper($additional_class = '')
  {

    $html = '';
    $html .= '<li class="' . $this->get_single_wrapper_classes($additional_class) . '" data-container="' . $this->type . '" >';

    if ($this->editable) {
      # Edit button: to be added, a button to control container type, label, key, css classes or any extra options related
      # <a href="#" class="container_control" data-action="edit"><span class="dashicons dashicons-edit"></span></a>
      $html .= '
         <div class="container_controls disable-sorting">
   				<div class="controls right">
   					<a href="#" class="container_control" data-action="delete"><span class="dashicons dashicons-trash"></span></a>
   					<a href="#" class="container_control" data-action="layout"><span class="dashicons dashicons-grid-view"></span></a>
   				</div>
   			</div>';
    }

    // Here goes the columns and the content of each column
    $html .= '{MF_WRAPPER_CONTENT}';

    $html .= '</li>';

    return $html;
  }

  /**
   *  Get the container inner columns
   *
   * @since    1.0.0
   * @param string $additional_class
   * @return string
   */
  public function get_single_column($width = 100, $additional_class = '')
  {

    # Make sure the `$additional_class` is turned into an array()
    if (!is_array($additional_class) && empty($additional_class)) {
      $additional_class = array();
    } elseif (!empty($additional_class)) {
      $additional_class = array($additional_class);
    }

    if (!$this->editable) {
      # Add classes conditionally
      if ($width == 100) {
        $additional_class[] = 'mfcol-size-full';
      } elseif ($width == 75) {
        $additional_class[] = 'mfcol-size-third';
      } elseif ($width == 50) {
        $additional_class[] = 'mfcol-size-half';
      } elseif ($width == 25) {
        $additional_class[] = 'mfcol-size-fourth';
      } else {
        $additional_class[] = 'mfcol-size-custom';
      }
    }

    # Add styles if needed
    $style_attr = !empty($width) ? ' style="width:' . (int) $width . '%"' : '';

    $html = '';
    $html .= '<ul class="' . $this->get_column_classes($additional_class) . '"' . $style_attr . '>';
    $html .= '{MF_COLUMN_CONTENT}';
    $html .= '</ul>';

    return $html;
  }

  /**
   *  Get the container css classes.
   *
   * @since    1.0.0
   * @param string $additional_class
   * @return string
   */
  public function get_single_wrapper_classes($additional_class = '')
  {

    $classes     = array(
      'mf-' . $this->type . '-container',
    );
    $classes[]   = 'mf_wrapper'; # Main CSS Class added to all containers

    if ($this->editable) {
      $classes[]   = 'mf_wrapper_preview'; # Preview CSS Class added only on the editor view
    }

    if (!empty($additional_class)) {
      if (is_array($additional_class)) {
        $classes = array_merge($classes, $additional_class);
      } else {
        $classes[]   = $additional_class;
      }
    }

    return implode(' ', $classes);
  }
  /**
   *  Get the columns css classes.
   *
   * @since    1.0.0
   * @param string $additional_class
   * @return string
   */
  public function get_column_classes($additional_class = '')
  {

    $classes     = array(
      'mf-' . $this->type . '-col',
    );
    $classes[]   = 'mf_col'; # Main CSS Class added to all columns

    if ($this->editable) {
      $classes[]   = 'mfcol_preview'; # Preview CSS Class added only on the editor view
    }

    if (!empty($additional_class)) {
      if (is_array($additional_class)) {
        $classes = array_merge($classes, $additional_class);
      } else {
        $classes[]   = $additional_class;
      }
    }

    return implode(' ', $classes);
  }


  /**
   * Insert any defined content inside the wrapper
   *
   * @since    1.0.0
   * @param  string $content
   * @return string
   */
  public function get_wrapper_output($content = '', $count = 1)
  {

    // If no content assigned, assign a single column as the content
    if (empty($content)) {
      $content = $this->get_column_output();
    }

    $wrapper = $this->get_single_wrapper('mf-' . $count . '-cols');

    return str_replace('{MF_WRAPPER_CONTENT}', $content, $wrapper);
  }

  /**
   * Insert any defined content inside the column
   *
   * @since    1.0.0
   * @param  string $content
   * @return string
   */
  public function get_column_output($width = 100, $content = '')
  {

    $column = $this->get_single_column($width);

    return str_replace('{MF_COLUMN_CONTENT}', $content, $column);
  }
}
