<?php

/**
 * @link       https://wpali.com
 * @since      1.0.0
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/admin/partials/fields
 */

/**
 * Textarea field type class
 *
 * @package    Mega_Forms
 * @subpackage Mega_Forms/admin/partials/fields
 * @author     ALI KHALLAD <ali@wpali.com>
 */

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class MegaForms_textarea extends MF_Field
{

	public $type = 'textarea';
	public $editorSettings = array(
		'general' => array(
			'field_max_length',
		),
	);


	public function get_field_title()
	{
		return esc_attr__('Paragraph Text', 'megaforms');
	}

	public function get_field_icon()
	{
		return 'mega-icons-paragraph';
	}

	public function get_field_display($value = null)
	{

		# Define arguements array and pass required arguements
		$args = $this->build_field_display_args();
		$args['value'] = $value;

		if (isset($this->field['max_length'])) {
			$args['attributes']['maxlength'] = $this->field['max_length'];
		}

		if (isset($this->field['rows'])) {
			$args['rows'] = $this->field['rows'];
		}

		# retrieve and return the input markup
		$input = mfinput('textarea', $args, $this->is_editor);

		return $input;
	}
	/**********************************************************************
	 ********************** Fields Options Markup *************************
	 **********************************************************************/

	/**
	 * Returns the display for field placeholder.
	 *
	 * @return string
	 */
	protected function field_placeholder()
	{
		$label = __('Placeholder', 'megaforms');
		$desc = __('Use this to specify a short hint that describes the expected value of this field ', 'megaforms');
		$field_key = 'field_placeholder';

		$args['name'] 				= $field_key;
		$args['id'] 					= $this->get_field_key('options', $field_key);
		$args['label'] 				= $label;
		$args['after_label'] 	= $this->get_description_tip_markup($label, $desc);
		$args['value'] 				= $this->get_setting_value($field_key);
		$args['onchange_preview'] 	= $this->get_js_helper_rules('textarea', 'update_placeholder');

		$input = mfinput('text', $args, true);
		return $input;
	}

	/**
	 * Returns the default value field markup .
	 *
	 * @return string
	 */
	protected function field_default()
	{

		$label = __('Default Value', 'megaforms');
		$desc = __('Use this to pre-populate this field value.', 'megaforms');
		$field_key = 'field_default';

		$args['id'] 					= $this->get_field_key('options', $field_key);
		$args['label'] 				= $label;
		$args['after_label'] 	= $this->get_description_tip_markup($label, $desc);
		$args['value'] 				= $this->get_setting_value($field_key);
		$args['onchange_preview'] 	= $this->get_js_helper_rules('textarea', 'update_value');

		$input = mfinput('text', $args, true);
		return $input;
	}
	/**
	 * Returns the markup for max-length option.
	 *
	 * @return string
	 */
	protected function field_max_length()
	{

		$label 		= __('Maximum Length', 'megaforms');
		$field_key = 'max_length';
		$desc 		= __('Specify the maximum number of characters allowed in this field.', 'megaforms');

		$args['inputType'] 		= 'number';
		$args['id'] 					= $this->get_field_key('options', $field_key);
		$args['label'] 				= $label;
		$args['after_label'] 	= $this->get_description_tip_markup($label, $desc);
		$args['value'] 				= $this->get_setting_value($field_key);

		$input = mfinput('text', $args, true);
		return $input;
	}
	/**********************************************************************
	 ********************* Validation && Sanitazation *********************
	 **********************************************************************/
	public function validate($value, $context = '')
	{

		$fieldLabel = $this->get_setting_value('field_label');
		$maxLength = $this->get_setting_value('max_length');

		if (!empty($maxLength)) {
			if (strlen($value) > $maxLength) {
				return array(
					'notice' => sprintf(__('The entered %s is too long.', 'megaforms'), $fieldLabel),
					'notice_code' => 'invalid_length',
				);
			}
		}

		return true;
	}
	public function sanitize_settings()
	{

		$sanitized = parent::sanitize_settings();

		$max_length = $this->get_setting_value('max_length');

		$sanitized['max_length'] = !empty($max_length) ? (int) $max_length : '';

		return $sanitized;
	}
}

MF_Fields::register(new MegaForms_textarea());
